package api

import (
	"context"
	"net"
	"witness/core"

	grpc_prometheus "github.com/grpc-ecosystem/go-grpc-prometheus"
	witnessv1 "github.com/odysseus/odysseus-protocol/gen/proto/go/witness/v1"
	log "github.com/sirupsen/logrus"
	"google.golang.org/grpc"
)

type Server struct {
	witnessv1.UnimplementedWitnessServiceServer
	w *core.Witness
}

func (s *Server) WitnessStatus(ctx context.Context, req *witnessv1.WitnessStatusRequest) (resp *witnessv1.WitnessStatusResponse, err error) {
	return nil, nil
}

func (s *Server) PushProof(ctx context.Context, req *witnessv1.PushProofRequest) (resp *witnessv1.PushProofResponse, err error) {
	return resp, nil
}

func (s *Server) GetMinerProof(ctx context.Context, req *witnessv1.GetMinerProofRequest) (*witnessv1.GetMinerProofResponse, error) {
	return nil, nil
}

func (s *Server) GetProof(ctx context.Context, req *witnessv1.GetProofRequest) (*witnessv1.GetProofResponse, error) {
	return nil, nil
}

func (s *Server) GetWithdrawProof(ctx context.Context, req *witnessv1.GetWithdrawProofRequest) (*witnessv1.GetWithdrawProofResponse, error) {
	return nil, nil
}

func StartGRPC(listenAddress string, w *core.Witness) {
	ln, err := net.Listen("tcp", listenAddress)
	if err != nil {
		log.WithError(err).Errorf("failed to listen on %s", listenAddress)
		return
	}

	log.WithField("listen", listenAddress).Info("start gRPC server")
	server := grpc.NewServer(grpc.MaxRecvMsgSize(1024*1024*1024), grpc.MaxSendMsgSize(1024*1024*1024))
	witnessv1.RegisterWitnessServiceServer(server, &Server{w: w})
	grpc_prometheus.Register(server)
	err = server.Serve(ln)
	if err != nil {
		log.WithError(err).Error("failed to serve")
		return
	}
	server.Stop()
}
